/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.util;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import java.util.Collection;
import nl.cloudfarming.client.field.model.TreatmentZone;

/**
 * Util class for geometries
 * Can provide the envelope for a collection of treatmentzones
 * Can provide the cellWidth and height for a certain envelope an grid columns rows
 * 
 * @author Merijn Zengers
 */
public class GeometryUtil {
    
    /**
     * Calculate grid cell width size
     *
     * @param envelope
     * @param nrOfGridColumns
     * @return
     */
    public static double getCellWidth(Geometry envelope, int nrOfGridColumns) {
        double envelopeMinX = getMinX(envelope);
        double envelopeMaxX = getMaxX(envelope);
        double width = envelopeMaxX - envelopeMinX;
        return width / nrOfGridColumns;
    }

    /**
     * Calculate grid cell height size
     *
     * @param envelope the envelope to calculate for
     * @param nrOfGridRows the number of rows in the grid
     * @return 
     */
    public static double getCellHeight(Geometry envelope, int nrOfGridRows) {
        double envelopeMinY = getMinY(envelope);
        double envelopeMaxY = getMaxY(envelope);
        double height = envelopeMaxY - envelopeMinY;
        return height / nrOfGridRows;
    }

    /**
     * Calculates the enveloper of the specified treatmentZones as a whole
     *
     * @param treatmentZones TreatmentZone to calculate envelope from
     * @return The envelope as a Geometry
     */
    public static Geometry getEnvelope(Collection<TreatmentZone> treatmentZones) {
        // Determine total envelope
        Geometry[] geometries = new Geometry[treatmentZones.size()];
        int counter = 0;
        for (TreatmentZone treatmentZone : treatmentZones) {
            geometries[counter] = treatmentZone.getGeometry();
            counter++;
        }
        GeometryCollection collection = new GeometryCollection(geometries, geometries[0].getFactory());
        return collection.getEnvelope();
    }
    
    /**
     * Get the largest X coordinate of the specified geometry
     * @param geometry Geometry to calculate largest X coordinate from
     * @return largest X in the geometry
     */
    public static double getMaxX(Geometry geometry){
        double maxX = geometry.getCoordinates()[0].x;
        for(Coordinate coordinate : geometry.getCoordinates()){
            if(coordinate.x > maxX){
                maxX = coordinate.x;
            }
        }
        return maxX;
    }
    
    /**
     * Get the smallest X coordinate of the specified geometry
     * @param geometry Geometry to calculate smallest X value from
     * @return smallest X in the geometry
     */
    public static double getMinX(Geometry geometry){
        double minX = geometry.getCoordinates()[0].x;
        for(Coordinate coordinate : geometry.getCoordinates()){
            if(coordinate.x < minX){
                minX = coordinate.x;
            }
        }
        return minX;
    }
    
    /**
     * Get the largest Y coordinate of the specified geometry
     * @param geometry Geometry to calculate largest Y coordinate from
     * @return largest Y in the geometry
     */
    public static double getMaxY(Geometry geometry){
        double maxY = geometry.getCoordinates()[0].y;
        for(Coordinate coordinate : geometry.getCoordinates()){
            if(coordinate.y > maxY){
                maxY = coordinate.y;
            }
            
        }
        return maxY;
    }
    
    /**
     * Get the smallest Y coordinate of the specified geometry
     * @param geometry Geometry to calculate smallest Y value from
     * @return smallest Y in the geometry
     */
    public static double getMinY(Geometry geometry){
        double minY = geometry.getCoordinates()[0].y;
        for(Coordinate coordinate : geometry.getCoordinates()){
            if(coordinate.y < minY){
                minY = coordinate.y;
            }
        }
        return minY;
    }
}
